@@ -2,7 +2,7 @@
 
 ;;; File: "_gambit#.scm"
 
-;;; Copyright (c) 1994-2015 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2018 by Marc Feeley, All Rights Reserved.
 
 ;;;============================================================================
 
@@ -75,11 +75,11 @@
 
 ;; Special objects.
 
-(##define-macro (macro-absent-obj)   `',c#absent-object) ;;(##type-cast -6 2))
-(##define-macro (macro-unbound1-obj) `',(##type-cast -7 2))
-(##define-macro (macro-unbound2-obj) `',(##type-cast -8 2))
-(##define-macro (macro-unused-obj)   `',(##type-cast -14 2))
-(##define-macro (macro-deleted-obj)  `',(##type-cast -15 2))
+(##define-macro (macro-absent-obj)   `',c#absent-object) ;;(##type-cast -6 (##type #f)))
+(##define-macro (macro-unbound1-obj) `',(##type-cast -7 (##type #f)))
+(##define-macro (macro-unbound2-obj) `',(##type-cast -8 (##type #f)))
+(##define-macro (macro-unused-obj)   `',(##type-cast -14 (##type #f)))
+(##define-macro (macro-deleted-obj)  `',(##type-cast -15 (##type #f)))
 
 ;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
@@ -137,48 +137,10 @@
     `(##vector-ref ,struct ,index)
     `(##vector-set! ,struct ,index ,@val)))
 
-;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-;; Symbol objects
-
-;; A symbol is represented by an object vector of length 4
-;; slot 0 = symbol name (a string or a fixnum <n> for a symbol named "g<n>")
-;; slot 1 = hash code (non-negative fixnum)
-;; slot 2 = link to next symbol in symbol table (#f for uninterned)
-;; slot 3 = pointer to corresponding global variable (0 if none exists)
-
-(##define-macro (macro-make-uninterned-symbol name hash)
-  `(##subtype-set!
-    (##vector ,name ,hash #f 0)
-    (macro-subtype-symbol)))
-
-(##define-macro (macro-symbol-name s)        `(macro-slot 0 ,s))
-(##define-macro (macro-symbol-name-set! s x) `(macro-slot 0 ,s ,x))
-(##define-macro (macro-symbol-hash s)        `(macro-slot 1 ,s))
-(##define-macro (macro-symbol-hash-set! s x) `(macro-slot 1 ,s ,x))
-(##define-macro (macro-symbol-next s)        `(macro-slot 2 ,s))
-(##define-macro (macro-symbol-next-set! s x) `(macro-slot 2 ,s ,x))
-
-;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-;; Keyword objects
-
-;; A keyword is represented by an object vector of length 3
-;; slot 0 = keyword name (a string or a fixnum <n> for a keyword named "g<n>")
-;; slot 1 = hash code (non-negative fixnum)
-;; slot 2 = link to next keyword in keyword table (#f for uninterned)
-
-(##define-macro (macro-make-uninterned-keyword name hash)
-  `(##subtype-set!
-    (##vector ,name ,hash #f)
-    (macro-subtype-keyword)))
-
-(##define-macro (macro-keyword-name k)        `(macro-slot 0 ,k))
-(##define-macro (macro-keyword-name-set! k x) `(macro-slot 0 ,k ,x))
-(##define-macro (macro-keyword-hash k)        `(macro-slot 1 ,k))
-(##define-macro (macro-keyword-hash-set! k x) `(macro-slot 1 ,k ,x))
-(##define-macro (macro-keyword-next k)        `(macro-slot 2 ,k))
-(##define-macro (macro-keyword-next-set! k x) `(macro-slot 2 ,k ,x))
+(##define-macro (macro-struct-slot index struct . val)
+  (if (null? val)
+    `(##unchecked-structure-ref ,struct ,index #f #f)
+    `(##unchecked-structure-set! ,struct ,@val ,index #f #f)))
 
 ;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
@@ -223,7 +185,7 @@
 ;; slot 5 = value of entry #0
 
 (##define-macro (macro-gc-hash-table-nb-entries ht)
-  `(##fxarithmetic-shift-right
+  `(##fxwraplogical-shift-right
     (##fx- (##vector-length ,ht) (macro-gc-hash-table-key0))
     1))
 
@@ -310,45 +272,36 @@
 (##define-macro (macro-continuation-denv-set! c x)
   `(##continuation-denv-set! ,c ,x))
 
-;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-;; Promise objects
-
-;; A promise is represented by an object vector of length 2
-;; slot 0 = thunk
-;; slot 1 = result (eq? to promise object itself when not yet determined)
-
-(##define-macro (macro-make-promise thunk)
-  `(##make-promise thunk))
-
-(##define-macro (macro-promise-thunk p)         `(macro-slot 0 ,p))
-(##define-macro (macro-promise-thunk-set! p x)  `(macro-slot 0 ,p ,x))
-(##define-macro (macro-promise-result p)        `(macro-slot 1 ,p))
-(##define-macro (macro-promise-result-set! p x) `(macro-slot 1 ,p ,x))
-
-;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-
-;; Foreign objects
-
-;; A foreign object is represented by an object vector of length 3
-;; slot 0 = tags
-;; slot 1 = C pointer to release function
-;; slot 2 = C pointer to foreign data
-
-(##define-macro (macro-foreign-tags f)         `(macro-slot 0 ,f))
-(##define-macro (macro-foreign-tags-set! f x)  `(macro-slot 0 ,f ,x))
-
 ;;;----------------------------------------------------------------------------
 
+(macro-define-syntax macro-target
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_)
+       (let ((target
+              (let* ((co
+                      (##global-var-ref
+                       (##make-global-var '##compilation-options)))
+                     (comp-opts
+                      (if (##unbound? co) '() co))
+                     (t
+                      (assq 'target comp-opts)))
+                (if t (cadr t) 'C))))
+         (datum->syntax stx `',target))))))
+
 (macro-define-syntax macro-case-target
   (lambda (stx)
     (syntax-case stx (else)
       ((_ . clauses)
-       (let ((target (if (and (pair? ##compilation-options)
-                              (pair? (car ##compilation-options)))
-                         (let ((t (assq 'target ##compilation-options)))
-                           (if t (cadr t) 'c))
-                         'c)))
+       (let ((target
+              (let* ((co
+                      (##global-var-ref
+                       (##make-global-var '##compilation-options)))
+                     (comp-opts
+                      (if (##unbound? co) '() co))
+                     (t
+                      (assq 'target comp-opts)))
+                (if t (cadr t) 'C))))
          (let loop ((clauses (syntax->list #'clauses)))
            (if (pair? clauses)
                (syntax-case (car clauses) (else)
@@ -360,55 +313,72 @@
                       (loop (cdr clauses)))))
                #'(begin))))))))
 
-(macro-define-syntax macro-if-forces
+(macro-define-syntax macro-if2-compilation-option
   (lambda (stx)
     (syntax-case stx ()
-      ((_ forces noforces)
-       (if ((if (and (pair? ##compilation-options)
-                     (pair? (car ##compilation-options)))
-                assq
-                memq)
-            'force
-            ##compilation-options)
+      ((_ option have-option dont-have-option)
+       (if (let* ((co
+                   (##global-var-ref
+                    (##make-global-var '##compilation-options)))
+                  (comp-opts
+                   (if (##unbound? co) '() co)))
+             (assq (syntax->datum #'option) comp-opts))
 
-           #'forces
+           #'have-option
 
-           #'noforces)))))
+           #'dont-have-option)))))
+
+(macro-define-syntax macro-if-compilation-option
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ option have-option)
+       #'(macro-if2-compilation-option option have-option (begin)))
+      ((_ option have-option dont-have-option)
+       #'(macro-if2-compilation-option option have-option dont-have-option)))))
+
+(macro-define-syntax macro-if-not-compilation-option
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ option dont-have-option)
+       #'(macro-if2-compilation-option option (begin) dont-have-option))
+      ((_ option dont-have-option have-option)
+       #'(macro-if2-compilation-option option have-option dont-have-option)))))
+
+(macro-define-syntax macro-if-auto-forcing
+  (lambda (stx)
+    (syntax-case stx ()
+      ((_ forcing noforcing)
+       #'(cond-expand
+          (enable-auto-forcing
+           forcing)
+          (else
+           noforcing))))))
 
 (macro-define-syntax macro-force-vars
   (lambda (stx)
     (syntax-case stx ()
       ((_ vars expr)
-       (if ((if (and (pair? ##compilation-options)
-                     (pair? (car ##compilation-options)))
-                assq
-                memq)
-            'force
-            ##compilation-options)
-
-           (syntax-case (datum->syntax
-                         #'vars
-                         (map (lambda (x) `(,x (##force ,x)))
-                              (syntax->list #'vars)))
-               ()
-             (bindings #'(let bindings expr)))
-
-           #'expr)))))
+       (syntax-case (datum->syntax
+                     #'vars
+                     (map (lambda (x) `(,x (##force ,x)))
+                          (syntax->list #'vars)))
+           ()
+         (bindings
+          #'(cond-expand
+             (enable-auto-forcing
+              (let bindings expr))
+             (else
+              expr))))))))
 
 (macro-define-syntax macro-if-checks
   (lambda (stx)
     (syntax-case stx ()
       ((_ checks nochecks)
-       (if ((if (and (pair? ##compilation-options)
-                     (pair? (car ##compilation-options)))
-                assq
-                memq)
-            'check
-            ##compilation-options)
-
-           #'checks
-
-           #'nochecks)))))
+       #'(cond-expand
+          (enable-type-checking
+           checks)
+          (else
+           nochecks))))))
 
 (macro-define-syntax macro-no-force
   (lambda (stx)
